write_text

Function write_text 

pub fn write_text(graph: &RelationGraph, path: &Path) -> Result<()>
Creates a new export file without ever opening an existing target for truncation. create_new makes the non-overwrite guarantee atomic with respect to another process creating the same path concurrently.