pub fn write_text(graph: &RelationGraph, path: &Path) -> Result<()>Expand description
Creates a new export file without ever opening an existing target for
truncation. create_new makes the non-overwrite guarantee atomic with
respect to another process creating the same path concurrently.